<?php

declare(strict_types=1);

namespace Drupal\site_verify\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigException;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\site_verify\Entity\SiteVerificationType;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Add or Edit SiteVerification entity.
 */
class SiteVerificationForm extends EntityForm {

  use AutowireTrait;

  /**
   * Creates a new SiteVerificationForm object.
   */
  public function __construct(
    protected AccountInterface $currentUser,
    #[Autowire(service: 'cache.render')]
    protected CacheBackendInterface $renderCache,
    protected RouteBuilderInterface $routeBuilder,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    /** @var \Drupal\site_verify\Entity\SiteVerificationInterface $site_verification */
    $site_verification = $this->entity;

    $form['identity'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Identity'),
    ];

    $form['identity']['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#description' => $this->t('Label for Site Verification.'),
      '#maxlength' => 255,
      '#required' => TRUE,
      '#default_value' => $site_verification->label(),
    ];

    $form['identity']['id'] = [
      '#type' => 'machine_name',
      '#machine_name' => [
        'exists' => [$this, 'exists'],
        'source' => ['identity', 'label'],
      ],
      '#default_value' => $site_verification->id(),
      '#disabled' => !$site_verification->isNew(),
    ];

    $form['identity']['description'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Description'),
      '#description' => $this->t('Admin description or note.'),
      '#maxlength' => 255,
      '#default_value' => $site_verification->getDescription(),
    ];

    $form['identity']['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Status'),
      '#default_value' => $site_verification->status(),
    ];

    $form['content'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Verification'),
    ];

    $form['content']['type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Verification type'),
      '#description' => $this->t(
        'Meta tags are included only on the <a href="@frontpage">front page</a>. Files are served from the site root.', [
          '@frontpage' => Url::fromRoute('<front>')->toString(),
        ]
      ),
      '#options' => [
        SiteVerificationType::Meta->value => $this->t('Meta tag'),
        SiteVerificationType::File->value => $this->t('File'),
      ],
      '#required' => TRUE,
      '#default_value' => $site_verification->getType()->value,
    ];

    // Default value of type can be set from query parameter on new entities.
    if ($this->entity->isNew() && ($query = $this->getRequest()->query)->has('type')) {
      // Only use the value if it's valid. Comparison is case sensitive.
      try {
        $type = SiteVerificationType::from($query->get('type'));
        $form['content']['type']['#default_value'] = $type->value;
      }
      catch (\ValueError $e) {
        // Emit simple warning. Do not include user input as not useful.
        $this->messenger()
          ->addWarning('Invalid type provided in query is ignored.');
      }
    }

    // Remove file handling ability if user does not have permission. User also
    // cannot edit existing File based verifications and will not see this form.
    if (!$this->currentUser->hasPermission('manage file based site verifications')) {
      $form['content']['type']['#default_value'] = SiteVerificationType::Meta->value;
      unset($form['content']['type']['#options'][SiteVerificationType::File->value]);
      $form['content']['type']['#description'] .= ' ' . $this->t(
        'Only Meta type available as Manage file type site verifications permission not granted to your role.'
      );
    }

    $form['content']['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Meta/File name'),
      '#description' => $this->t('The meta tag name part, or the filename off the site root.'),
      '#required' => TRUE,
      '#default_value' => $site_verification->getName(),
    ];

    $form['content']['content'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Meta/File content'),
      '#description' => $this->t('Contents of the meta tag or file contents served as plain/text.'),
      '#wysiwyg' => FALSE,
      '#default_value' => $site_verification->getContent(),
    ];

    return parent::form($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    // Add the cancel link.
    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $this->buildCancelLinkUrl(),
      '#attributes' => ['class' => ['button']],
      '#weight' => 15,
    ];

    return $form;
  }

  /**
   * Builds the cancel link url for the form.
   *
   * @return \Drupal\Core\Url
   *   Cancel url
   */
  protected function buildCancelLinkUrl() {
    $query = $this->getRequest()->query;

    if ($query->has('destination')) {
      $options = UrlHelper::parse($query->get('destination'));
      $url = Url::fromUserInput('/' . ltrim($options['path'], '/'), $options);
    }
    else {
      $url = Url::fromRoute('entity.site_verification.collection');
    }

    return $url;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Entity has already been rebuilt using form input. Check for violations.
    if ($violations = $this->entity->validate()) {
      foreach ($violations as $violation) {
        $form_state->setErrorByName(
          $violation->getPropertyPath(),
          $violation->getMessage()
        );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    // Set the menu to be rebuilt.
    $this->routeBuilder->rebuild();

    // Goal: Flush the front page cache. Actual: Flush all render.
    $this->renderCache->invalidateAll();

    $this->messenger()->addStatus($this->t('Site Verification @name @state.', [
      '@name' => $this->entity->label(),
      '@state' => $this->entity->isNew()
        ? $this->t('saved')
        : $this->t('updated'),
    ]));

    // Redirect back to the collection. Destination parameter will override.
    $form_state->setRedirect('entity.site_verification.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // Belt & brace: This validation error should never be reached.
    $violations = $this->entity->validate();
    if (empty($violations)) {
      return $this->entity->save();
    }

    throw new ConfigException('Site Verification entity is not valid.');
  }

  /**
   * Unique machine name callback.
   */
  public function exists($id) {
    $site_verification_storage = $this->entityTypeManager
      ->getStorage('site_verification');
    $entity = $site_verification_storage->getQuery()
      ->condition('id', $id)
      ->accessCheck(FALSE)
      ->execute();
    return (bool) $entity;
  }

}
